; Use TIM1T as temporary storage
; By Omegamatrix
;
; This program uses no ram! What is more unique is looped 48 bit
; static displays are used taking advantage of TIM1T to hold a
; digit. You just have to add the number of cycles between the
; storing of TIM1T and loading of INTIM to your value, and you
; have a temporary storage location. In this demo those cycles
; are added to the graphic values beforehand.
;
; In this demo the stack pointer holds the loop count. Using the stack
; pointer for storage has been utilized in the past by many other
; people. Other possible storage places not in zero page ram include
; using SWCHB once the DDR has been configured. Two of SWCHB's bits
; are never used, but all eight bits can be made available for
; storage when not reading switches. Other than that the status
; register's interrupt bit can be used as a 1 bit storage since it
; is not used by the 2600, and the 7800 locks into 2600 mode before
; a game runs on it.
;
;
; Finally, the technique of using TIM1T in standard 48 bit routine with
; (indirect),Y addressing. You can quickly account for the extra cycles
; the timer gobbles up like so:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; standard 48 bit display
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;    ldy    #7
;    sty    loopCount
;.loopStandard48bit:
;    lda    (gfxPtrs+10),Y        ;5  @69
;    sta    tempGfx               ;3  @72
;    sta    WSYNC                 ;3
;---------------------------------------
;    lda    (gfxPtrs),Y           ;5  @5
;    sta    GRP0                  ;3  @8
;    lda    (gfxPtrs+2),Y         ;5  @13
;    sta    GRP1                  ;3  @16
;    lda    (gfxPtrs+4),Y         ;5  @21
;    sta    GRP0                  ;3  @24
;    lax    (gfxPtrs+8),Y         ;5  @29
;    lda    (gfxPtrs+6),Y         ;5  @34
;    ldy    tempGfx               ;3  @37
;    nop                          ;2  @39
;    nop                          ;2  @41
;    sta    GRP1                  ;3  @44
;    stx    GRP0                  ;3  @47
;    sty    GRP1                  ;3  @50
;    sta    GRP0                  ;3  @53
;    dec    loopCount             ;5  @58
;    ldy    loopCount             ;3  @61
;    bpl    .loopStandard48bit    ;2 @63/64
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Using TIM1T
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;    ldy    #7
;    sty    loopCount
;.loopStandard48bit:
;    lax    (gfxPtrs+10),Y        ;5  @69      use LAX before SBX to have A = X
;    sbx    #EXTRA_CYCLES         ;2  @71      EXTRA_CYCLES = -38, use a negative sign to make SBX add! Carry doesn't matter for SBX
;    sta    TIM1T                 ;4  @75
;---------------------------------------
;    lda    (gfxPtrs),Y           ;5  @4
;    sta    GRP0                  ;3  @7
;    lda    (gfxPtrs+2),Y         ;5  @12
;    sta    GRP1                  ;3  @15
;    lda    (gfxPtrs+4),Y         ;5  @20
;    sta    GRP0                  ;3  @23
;    lax    (gfxPtrs+8),Y         ;5  @28
;    lda    (gfxPtrs+6),Y         ;5  @33
;    ldy    INTIM                 ;4  @37
;    nop                          ;2  @39
;    nop                          ;2  @41
;    sta    GRP1                  ;3  @44
;    stx    GRP0                  ;3  @47
;    sty    GRP1                  ;3  @50
;    sta    GRP0                  ;3  @53
;    dec    loopCount             ;5  @58
;    ldy    loopCount             ;3  @61
;    bpl    .loopStandard48bit    ;2 @63/64



      processor 6502

VSYNC   =  $00
VBLANK  =  $01
WSYNC   =  $02
NUSIZ0  =  $04
NUSIZ1  =  $05
COLUP0  =  $06
COLUP1  =  $07
COLUPF  =  $08
COLUBK  =  $09
CTRLPF  =  $0A
REFP0   =  $0B
REFP1   =  $0C
RESP0   =  $10
RESP1   =  $11
AUDV0   =  $19
AUDV1   =  $1A
GRP0    =  $1B
GRP1    =  $1C
ENAM0   =  $1D
ENAM1   =  $1E
ENABL   =  $1F
HMP0    =  $20
HMP1    =  $21
VDELP0  =  $25
VDELP1  =  $26
HMOVE   =  $2A
INTIM   =  $0284
TIM1T   =  $0294
TIM64T  =  $0296

;delays for the Timer grahics
CYCLES       = 39
FIRST_CYCLE  = 93  ; First line gets stored in the timer a little sooner...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      MAIN PROGRAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG Code
       ORG $F000

START:

;one time initializations...
    sta   WSYNC
;---------------------------------------
    lda    #0                    ;2  @2
    sta    AUDV0                 ;3  @5
    sta    AUDV1                 ;3  @8
    sta    ENAM0                 ;3  @11
    sta    ENAM1                 ;3  @14
    sta    ENABL                 ;3  @17
    sta    GRP0                  ;3  @20
    sta    GRP1                  ;3  @23
    sta    REFP0                 ;3  @26
    sta    REFP1                 ;3  @29
    sta    CTRLPF                ;3  @32
    lda    #$B4                  ;2  @34
    sta    RESP0                 ;3  @37
    sta    RESP1                 ;3  @40
    sta    COLUBK                ;3  @43
    sta    COLUPF                ;3  @46
    lda    #$0D                  ;2  @48    white
    sta    COLUP0                ;3  @51
    sta    COLUP1                ;3  @54
    sta    VDELP0                ;3  @57    delay
    sta    VDELP1                ;3  @60
    lda    #$83                  ;2  @62
    sta    NUSIZ0                ;3  @65    right 8, 3 copies close
    sta    NUSIZ1                ;3  @68
    sta    HMP0                  ;3  @71
    lda    #$90                  ;2  @73    right 7
    sta    HMP1                  ;3  @76
;---------------------------------------
    sta    HMOVE




MainLoop:
    lda    #$0E
.loopVsync:
    sta    WSYNC
;---------------------------------------
    sta    VSYNC
    lsr
    bne    .loopVsync
    lda    #49
    sta    TIM64T

.waitVblank:
    lda    INTIM
    bne    .waitVblank
    sta    WSYNC
;---------------------------------------
    sta    VBLANK
    jmp    Kernel


       ORG $F100


Kernel:
    ldy    #71
.loopTop:
    sta    WSYNC
;---------------------------------------
    dey
    bne    .loopTop


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NO RAM WAS HARMED IN THE MAKING OF THIS ROM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


    ldx    #7                    ;2  @60
.loop_No_Ram_Was:
    txs                          ;2  @62
    lda    NRW_Six,X             ;4  @66
    sta    TIM1T                 ;4  @70
    sta    WSYNC                 ;3
;---------------------------------------
    lda    NRW_One,X             ;4  @4
    sta    GRP0                  ;3  @7
    lda    NRW_Two,X             ;4  @11
    sta    GRP1                  ;3  @14
    lda    NRW_Three,X           ;4  @18
    sta    GRP0                  ;3  @21
    lda    NRW_Four,X            ;4  @25
    ldy    NRW_Five,X            ;4  @29
    ldx    TIM1T                 ;4  @33
    nop                          ;2  @35
    nop                          ;2  @37
    nop                          ;2  @39
    nop                          ;2  @41
    sta    GRP1                  ;3  @44
    sty    GRP0                  ;3  @47
    stx    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    tsx                          ;2  @55
    dex                          ;2  @57
    bpl    .loop_No_Ram_Was      ;2 @59/60


    sta    WSYNC
    sta    WSYNC
    sta    WSYNC
    sta    WSYNC
    nop
    nop


    ldx    #7
.loop_Harmed_In:
    txs                          ;2  @62
    lda    HI_Six,X              ;4  @66
    sta    TIM1T                 ;4  @70
    sta    WSYNC                 ;3
;---------------------------------------
    lda    HI_One,X              ;4  @4
    sta    GRP0                  ;3  @7
    lda    HI_Two,X              ;4  @11
    sta    GRP1                  ;3  @14
    lda    HI_Three,X            ;4  @18
    sta    GRP0                  ;3  @21
    lda    HI_Four,X             ;4  @25
    ldy    HI_Five,X             ;4  @29
    ldx    TIM1T                 ;4  @33
    nop                          ;2  @35
    nop                          ;2  @37
    nop                          ;2  @39
    nop                          ;2  @41
    sta    GRP1                  ;3  @44
    sty    GRP0                  ;3  @47
    stx    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    tsx                          ;2  @55
    dex                          ;2  @57
    bpl    .loop_Harmed_In       ;2 @59/60


    sta    WSYNC
    sta    WSYNC
    sta    WSYNC
    sta    WSYNC
    nop
    nop


    ldx    #7
.loop_The_Making:
    txs                          ;2  @62
    lda    TM_Six,X              ;4  @66
    sta    TIM1T                 ;4  @70
    sta    WSYNC                 ;3
;---------------------------------------
    lda    TM_One,X              ;4  @4
    sta    GRP0                  ;3  @7
    lda    TM_Two,X              ;4  @11
    sta    GRP1                  ;3  @14
    lda    TM_Three,X            ;4  @18
    sta    GRP0                  ;3  @21
    lda    TM_Four,X             ;4  @25
    ldy    TM_Five,X             ;4  @29
    ldx    TIM1T                 ;4  @33
    nop                          ;2  @35
    nop                          ;2  @37
    nop                          ;2  @39
    nop                          ;2  @41
    sta    GRP1                  ;3  @44
    sty    GRP0                  ;3  @47
    stx    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    tsx                          ;2  @55
    dex                          ;2  @57
    bpl    .loop_The_Making      ;2 @59/60


    sta    WSYNC
    sta    WSYNC
    sta    WSYNC
    sta    WSYNC
    nop
    nop


    ldx    #7
.loop_Of_This_Rom:
    txs                          ;2  @62
    lda    OTR_Six,X             ;4  @66
    sta    TIM1T                 ;4  @70
    sta    WSYNC                 ;3
;---------------------------------------
    lda    OTR_One,X             ;4  @4
    sta    GRP0                  ;3  @7
    lda    OTR_Two,X             ;4  @11
    sta    GRP1                  ;3  @14
    lda    OTR_Three,X           ;4  @18
    sta    GRP0                  ;3  @21
    lda    OTR_Four,X            ;4  @25
    ldy    OTR_Five,X            ;4  @29
    ldx    TIM1T                 ;4  @33
    nop                          ;2  @35
    nop                          ;2  @37
    nop                          ;2  @39
    nop                          ;2  @41
    sta    GRP1                  ;3  @44
    sty    GRP0                  ;3  @47
    stx    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    tsx                          ;2  @55
    dex                          ;2  @57
    bpl    .loop_Of_This_Rom     ;2 @59/60



    ldy    #79
.loopBottom:
    sta    WSYNC
;---------------------------------------
    dey
    bne    .loopBottom

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      OVERSCAN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    lda    #2
    sta    WSYNC
;---------------------------------------
    sta    VBLANK
    lda    #27
    sta    TIM64T


.waitOverscan:
    lda    INTIM
    bne    .waitOverscan
    jmp    MainLoop



       ORG $FF00

NRW_One:
    .byte $00 ; |        | $FF00
    .byte $89 ; |X   X  X| $FF01
    .byte $8A ; |X   X X | $FF02
    .byte $8A ; |X   X X | $FF03
    .byte $9A ; |X  XX X | $FF04
    .byte $AA ; |X X X X | $FF05
    .byte $CA ; |XX  X X | $FF06
    .byte $89 ; |X   X  X| $FF07
NRW_Two:
    .byte $00 ; |        | $FF08
    .byte $82 ; |X     X | $FF09
    .byte $42 ; | X    X | $FF0A
    .byte $42 ; | X    X | $FF0B
    .byte $43 ; | X    XX| $FF0C
    .byte $42 ; | X    X | $FF0D
    .byte $42 ; | X    X | $FF0E
    .byte $83 ; |X     XX| $FF0F
NRW_Three:
    .byte $00 ; |        | $FF10
    .byte $52 ; | X X  X | $FF11
    .byte $52 ; | X X  X | $FF12
    .byte $52 ; | X X  X | $FF13
    .byte $9E ; |X  XXXX | $FF14
    .byte $52 ; | X X  X | $FF15
    .byte $52 ; | X X  X | $FF16
    .byte $8C ; |X   XX  | $FF17
NRW_Four:
    .byte $00 ; |        | $FF18
    .byte $88 ; |X   X   | $FF19
    .byte $88 ; |X   X   | $FF1A
    .byte $88 ; |X   X   | $FF1B
    .byte $A8 ; |X X X   | $FF1C
    .byte $A8 ; |X X X   | $FF1D
    .byte $D8 ; |XX XX   | $FF1E
    .byte $50 ; | X X    | $FF1F
NRW_Five:
    .byte $00 ; |        | $FF20
    .byte $29 ; |  X X  X| $FF21
    .byte $7D ; | XXXXX X| $FF22
    .byte $55 ; | X X X X| $FF23
    .byte $45 ; | X   X X| $FF24
    .byte $45 ; | X   X X| $FF25
    .byte $45 ; | X   X X| $FF26
    .byte $44 ; | X   X  | $FF27
NRW_Six:
    .byte $00 + CYCLES      ; |        | $FF28
    .byte $2E + CYCLES      ; |  X XXX | $FF29
    .byte $21 + CYCLES      ; |  X    X| $FF2A
    .byte $21 + CYCLES      ; |  X    X| $FF2B
    .byte $E6 + CYCLES      ; |XXX  XX | $FF2C
    .byte $28 + CYCLES      ; |  X X   | $FF2D
    .byte $28 + CYCLES      ; |  X X   | $FF2E
    .byte $C7 + FIRST_CYCLE ; |XX   XXX| $FF2F


HI_One:
    .byte $00 ; |        | $FF30
    .byte $4A ; | X  X X | $FF31
    .byte $4A ; | X  X X | $FF32
    .byte $4A ; | X  X X | $FF33
    .byte $7B ; | XXXX XX| $FF34
    .byte $4A ; | X  X X | $FF35
    .byte $4A ; | X  X X | $FF36
    .byte $49 ; | X  X  X| $FF37
HI_Two:
    .byte $00 ; |        | $FF38
    .byte $52 ; | X X  X | $FF39
    .byte $52 ; | X X  X | $FF3A
    .byte $52 ; | X X  X | $FF3B
    .byte $DC ; |XX XXX  | $FF3C
    .byte $52 ; | X X  X | $FF3D
    .byte $52 ; | X X  X | $FF3E
    .byte $9C ; |X  XXX  | $FF3F
HI_Three:
    .byte $00 ; |        | $FF40
    .byte $8B ; |X   X XX| $FF41
    .byte $8A ; |X   X X | $FF42
    .byte $8A ; |X   X X | $FF43
    .byte $AB ; |X X X XX| $FF44
    .byte $AA ; |X X X X | $FF45
    .byte $DA ; |XX XX X | $FF46
    .byte $53 ; | X X  XX| $FF47
HI_Four:
    .byte $00 ; |        | $FF48
    .byte $DC ; |XX XXX  | $FF49
    .byte $12 ; |   X  X | $FF4A
    .byte $12 ; |   X  X | $FF4B
    .byte $92 ; |X  X  X | $FF4C
    .byte $12 ; |   X  X | $FF4D
    .byte $12 ; |   X  X | $FF4E
    .byte $DC ; |XX XXX  | $FF4F
HI_Five:
    .byte $00 ; |        | $FF50
    .byte $07 ; |     XXX| $FF51
    .byte $02 ; |      X | $FF52
    .byte $02 ; |      X | $FF53
    .byte $02 ; |      X | $FF54
    .byte $02 ; |      X | $FF55
    .byte $02 ; |      X | $FF56
    .byte $07 ; |     XXX| $FF57
HI_Six:
    .byte $00 + CYCLES      ; |        | $FF58
    .byte $44 + CYCLES      ; | X   X  | $FF59
    .byte $44 + CYCLES      ; | X   X  | $FF5A
    .byte $44 + CYCLES      ; | X   X  | $FF5B
    .byte $4C + CYCLES      ; | X  XX  | $FF5C
    .byte $54 + CYCLES      ; | X X X  | $FF5D
    .byte $64 + CYCLES      ; | XX  X  | $FF5E
    .byte $44 + FIRST_CYCLE ; | X   X  | $FF5F


TM_One:
    .byte $00 ; |        | $FF60
    .byte $49 ; | X  X  X| $FF61
    .byte $49 ; | X  X  X| $FF62
    .byte $49 ; | X  X  X| $FF63
    .byte $4F ; | X  XXXX| $FF64
    .byte $49 ; | X  X  X| $FF65
    .byte $49 ; | X  X  X| $FF66
    .byte $E9 ; |XXX X  X| $FF67
TM_Two:
    .byte $00 ; |        | $FF68
    .byte $70 ; | XXX    | $FF69
    .byte $40 ; | X      | $FF6A
    .byte $40 ; | X      | $FF6B
    .byte $70 ; | XXX    | $FF6C
    .byte $40 ; | X      | $FF6D
    .byte $40 ; | X      | $FF6E
    .byte $70 ; | XXX    | $FF6F
TM_Three:
    .byte $00 ; |        | $FF70
    .byte $11 ; |   X   X| $FF71
    .byte $11 ; |   X   X| $FF72
    .byte $11 ; |   X   X| $FF73
    .byte $11 ; |   X   X| $FF74
    .byte $15 ; |   X X X| $FF75
    .byte $1F ; |   XXXXX| $FF76
    .byte $0A ; |    X X | $FF77
TM_Four:
    .byte $00 ; |        | $FF78
    .byte $4A ; | X  X X | $FF79
    .byte $4A ; | X  X X | $FF7A
    .byte $4A ; | X  X X | $FF7B
    .byte $7B ; | XXXX XX| $FF7C
    .byte $4A ; | X  X X | $FF7D
    .byte $4A ; | X  X X | $FF7E
    .byte $32 ; |  XX  X | $FF7F
TM_Five:
    .byte $00 ; |        | $FF80
    .byte $5D ; | X XXX X| $FF81
    .byte $49 ; | X  X  X| $FF82
    .byte $89 ; |X   X  X| $FF83
    .byte $09 ; |    X  X| $FF84
    .byte $89 ; |X   X  X| $FF85
    .byte $49 ; | X  X  X| $FF86
    .byte $5D ; | X XXX X| $FF87
TM_Six:
    .byte $00 + CYCLES      ; |        | $FF88
    .byte $26 + CYCLES      ; |  X  XX | $FF89
    .byte $29 + CYCLES      ; |  X X  X| $FF8A
    .byte $29 + CYCLES      ; |  X X  X| $FF8B
    .byte $2B + CYCLES      ; |  X X XX| $FF8C
    .byte $68 + CYCLES      ; | XX X   | $FF8D
    .byte $A9 + CYCLES      ; |X X X  X| $FF8E
    .byte $26 + FIRST_CYCLE ; |  X  XX | $FF8F


OTR_One:
    .byte $00 ; |        | $FF90
    .byte $64 ; | XX  X  | $FF91
    .byte $94 ; |X  X X  | $FF92
    .byte $94 ; |X  X X  | $FF93
    .byte $96 ; |X  X XX | $FF94
    .byte $94 ; |X  X X  | $FF95
    .byte $94 ; |X  X X  | $FF96
    .byte $67 ; | XX  XXX| $FF97
OTR_Two:
    .byte $00 ; |        | $FF98
    .byte $04 ; |     X  | $FF99
    .byte $04 ; |     X  | $FF9A
    .byte $04 ; |     X  | $FF9B
    .byte $04 ; |     X  | $FF9C
    .byte $04 ; |     X  | $FF9D
    .byte $04 ; |     X  | $FF9E
    .byte $0E ; |    XXX | $FF9F
OTR_Three:
    .byte $00 ; |        | $FFA0
    .byte $97 ; |X  X XXX| $FFA1
    .byte $92 ; |X  X  X | $FFA2
    .byte $92 ; |X  X  X | $FFA3
    .byte $F2 ; |XXXX  X | $FFA4
    .byte $92 ; |X  X  X | $FFA5
    .byte $92 ; |X  X  X | $FFA6
    .byte $97 ; |X  X XXX| $FFA7
OTR_Four:
    .byte $00 ; |        | $FFA8
    .byte $60 ; | XX     | $FFA9
    .byte $10 ; |   X    | $FFAA
    .byte $10 ; |   X    | $FFAB
    .byte $20 ; |  X     | $FFAC
    .byte $40 ; | X      | $FFAD
    .byte $40 ; | X      | $FFAE
    .byte $30 ; |  XX    | $FFAF
OTR_Five:
    .byte $00 ; |        | $FFB0
    .byte $49 ; | X  X  X| $FFB1
    .byte $4A ; | X  X X | $FFB2
    .byte $4A ; | X  X X | $FFB3
    .byte $72 ; | XXX  X | $FFB4
    .byte $4A ; | X  X X | $FFB5
    .byte $4A ; | X  X X | $FFB6
    .byte $71 ; | XXX   X| $FFB7
OTR_Six:
    .byte $00 + CYCLES      ; |        | $FFB8
    .byte $91 + CYCLES      ; |X  X   X| $FFB9
    .byte $51 + CYCLES      ; | X X   X| $FFBA
    .byte $51 + CYCLES      ; | X X   X| $FFBB
    .byte $51 + CYCLES      ; | X X   X| $FFBC
    .byte $55 + CYCLES      ; | X X X X| $FFBD
    .byte $5F + CYCLES      ; | X XXXXX| $FFBE
    .byte $8A + FIRST_CYCLE ; |X   X X | $FFBF


       ORG $FFFC

    .word START
    .word START